pub enum MoveAndSlideHitResponse {
Accept,
Ignore,
Abort,
}Expand description
Indicates how to handle a hit detected during MoveAndSlide::move_and_slide.
This is returned by the on_hit callback provided to MoveAndSlide::move_and_slide.
Variants
Accept
Accept the hit and continue the move and slide algorithm.
Ignore
Ignore the hit and continue the move and slide algorithm.
Note that the shape will still be moved up to the point of collision, but the velocity will not be modified to slide along the surface.
Abort
Ignore the hit and abort the move and slide algorithm.
Note that the shape will still be moved up to the point of collision, but no further movement or velocity modification will be performed.