pub fn project_velocity_bruteforce(v: Vector, normals: &[Dir]) -> VectorExpand description
Projects input velocity v onto the planes defined by the given normals.
This ensures that velocity does not point into any of the planes, but along them.
This is often used after MoveAndSlide::cast_move to ensure a character moved that way
does not try to continue moving into colliding geometry.
This is a brute-force implementation that tests all possible projections.
Consider using project_velocity for better performance on larger sets of normals.