pub struct DepenetrationConfig {
pub depenetration_iterations: usize,
pub max_depenetration_error: Scalar,
pub penetration_rejection_threshold: Scalar,
pub skin_width: Scalar,
}Expand description
Configuration for MoveAndSlide::depenetrate.
Fields
depenetration_iterations: usizeHow many iterations to use when performing depenetration.
Depenetration is an iterative process that solves penetrations for all planes,
until we either reached MoveAndSlideConfig::move_and_slide_iterations
or the accumulated error is less than MoveAndSlideConfig::max_depenetration_error.
To disable depenetration, set this to 0.
max_depenetration_error: ScalarThe target error to achieve when performing depenetration.
Depenetration is an iterative process that solves penetrations for all planes,
until we either reached MoveAndSlideConfig::move_and_slide_iterations
or the accumulated error is less than MoveAndSlideConfig::max_depenetration_error.
This is implicitly scaled by the PhysicsLengthUnit.
penetration_rejection_threshold: ScalarThe maximum penetration depth that is allowed for a contact to be resolved during depenetration.
This is used to reject invalid contacts that have an excessively high penetration depth, which can lead to clipping through geometry. This may be removed in the future once the collision errors in the underlying collision detection system are fixed.
This is implicitly scaled by the PhysicsLengthUnit.
skin_width: ScalarA minimal distance to always keep between the collider and any other colliders.
This exists to improve numerical stability and ensure that the collider never intersects anything. Set this to a small enough value that you don’t see visual artifacts but have good stability.
Increase the value if you notice your character getting stuck in geometry. Decrease it when you notice jittering, especially around V-shaped walls.
This is implicitly scaled by the PhysicsLengthUnit.