pub struct MoveAndSlideHitData<'a> {
pub entity: Entity,
pub distance: Scalar,
pub point: Vector,
pub normal: &'a mut Dir,
pub position: &'a mut Vector,
pub velocity: &'a mut Vector,
pub collision_distance: Scalar,
}Expand description
Data related to a hit during MoveAndSlide::move_and_slide.
Fields
entity: EntityThe entity of the collider that was hit by the shape.
distance: ScalarThe maximum distance that is safe to move in the given direction so that the collider
still keeps a distance of skin_width to the other colliders.
This is 0.0 when any of the following is true:
- The collider started off intersecting another collider.
- The collider is moving toward another collider that is already closer than
skin_width.
If you want to know the real distance to the next collision, use Self::collision_distance.
point: VectorThe hit point on the shape that was hit, expressed in world space.
normal: &'a mut DirThe outward surface normal on the hit shape at point, expressed in world space.
position: &'a mut VectorThe position of the collider at the time of the move and slide iteration.
velocity: &'a mut VectorThe velocity of the collider at the time of the move and slide iteration.
collision_distance: ScalarThe raw distance to the next collision, not respecting skin width.
To move the shape, use Self::distance instead.
Implementations
Sourceimpl<'a> MoveAndSlideHitData<'a>
impl<'a> MoveAndSlideHitData<'a>
Sourcepub fn intersects(&self) -> bool
pub fn intersects(&self) -> bool
Whether the collider started off already intersecting another collider when it was cast.
Note that this will be false if the collider was closer than skin_width, but not physically intersecting.