pub struct MoveAndSlideOutput {
pub position: Vector,
pub projected_velocity: Vector,
}Expand description
Output from MoveAndSlide::move_and_slide.
Fields
position: VectorThe final position of the character after move and slide.
Set your Transform::translation to this value.
projected_velocity: VectorThe final velocity of the character after move and slide.
This corresponds to the remaining velocity after the algorithm has slid along all contact surfaces. For example, if the character is trying to move to the right, but there is a ramp in its path, the projected velocity will point up the ramp, with reduced magnitude.
It is useful to store this value or apply it to LinearVelocity and use it as the input velocity
for the next frame’s call to MoveAndSlide::move_and_slide.
Note that if you apply this to LinearVelocity, it is recommended to use CustomPositionIntegration.
This ways, the character’s position is only updated via the move and slide algorithm,
and not also by the physics integrator.